@@ -19,28 +19,32 @@ import code
 import readline
 import __builtin__
 
-from LMIUtil import LMIPassByRef
-from LMIIndication import LMIIndication
-from LMIIndicationListener import LMIIndicationListener
-from LMIBaseClient import LMIBaseClient
-from LMIShellClient import LMIShellClient
-from LMIShellConfig import LMIShellConfig
-from LMIShellOptions import LMIShellOptions
-from LMISubscription import LMISubscription
-from LMICompleter import LMICompleter
-from LMIConnection import LMIConnection
-from LMIShellConfig import LMIShellConfig
-from LMIReturnValue import LMIReturnValue
-from LMIObjectFactory import LMIObjectFactory
+from lmi.shell.LMIUtil import LMIPassByRef
+from lmi.shell.LMIIndicationListener import LMIIndicationListener
+from lmi.shell.LMICIMXMLClient import LMICIMXMLClient
+from lmi.shell.LMIWSMANClient import LMIWSMANClient
+from lmi.shell.LMIShellClient import LMIShellClient
+from lmi.shell.LMIShellConfig import LMIShellConfig
+from lmi.shell.LMIShellOptions import LMIShellOptions
+from lmi.shell.LMISubscription import LMISubscription
+from lmi.shell.LMICompleter import LMICompleter
+from lmi.shell.LMIConnection import LMIConnection
+from lmi.shell.LMIShellConfig import LMIShellConfig
+from lmi.shell.LMIReturnValue import LMIReturnValue
+from lmi.shell.LMIObjectFactory import LMIObjectFactory
 
-from LMIConnection import connect as connect_internal
+from lmi.shell.LMIConnection import connect as connect_internal
 
-from LMIUtil import lmi_set_use_exceptions
-from LMIUtil import lmi_get_use_exceptions
-from LMIUtil import lmi_associators
-from LMIUtil import lmi_isinstance
+from lmi.shell.LMIExceptions import CIMError
+from lmi.shell.LMIExceptions import ConnectionError
+
+from lmi.shell.LMIUtil import lmi_set_use_exceptions
+from lmi.shell.LMIUtil import lmi_get_use_exceptions
+from lmi.shell.LMIUtil import lmi_associators
+from lmi.shell.LMIUtil import lmi_isinstance
+
+from lmi.shell.LMIHelper import LMIHelper
 
-from LMIHelper import LMIHelper
 
 class LMIConsole(code.InteractiveConsole):
     """
@@ -49,27 +53,29 @@ class LMIConsole(code.InteractiveConsole):
     SHELL_PS1 = "> "
     SHELL_PS2 = "... "
     DEFAULT_LOCALS = {
-        "__name__" : "__main__",
-        "LMIPassByRef" : LMIPassByRef,
-        "LMIIndication" : LMIIndication,
-        "LMIIndicationListener" : LMIIndicationListener,
-        "LMIReturnValue" : LMIReturnValue,
-        "LMIBaseClient" : LMIBaseClient,
-        "LMIShellClient" : LMIShellClient,
-        "LMIShellConfig" : LMIShellConfig,
-        "LMIShellOptions" : LMIShellOptions,
-        "LMISubscription" : LMISubscription,
-        "LMIConnection" : LMIConnection,
-        "LMINamespace" : LMIObjectFactory().LMINamespace,
-        "LMINamespaceRoot" : LMIObjectFactory().LMINamespaceRoot,
-        "LMIClass" : LMIObjectFactory().LMIClass,
-        "LMIInstance" : LMIObjectFactory().LMIInstance,
-        "LMIInstanceName" : LMIObjectFactory().LMIInstanceName,
-        "LMIMethod" : LMIObjectFactory().LMIMethod,
-        "use_exceptions" : lmi_set_use_exceptions,
-        "lmi_associators" : lmi_associators,
-        "lmi_isinstance" : lmi_isinstance,
-        "help" : LMIHelper()
+        "__name__": "__main__",
+        "LMIPassByRef": LMIPassByRef,
+        "LMIIndicationListener": LMIIndicationListener,
+        "LMIReturnValue": LMIReturnValue,
+        "LMICIMXMLClient": LMICIMXMLClient,
+        "LMIWSMANClient": LMIWSMANClient,
+        "LMIShellClient": LMIShellClient,
+        "LMIShellConfig": LMIShellConfig,
+        "LMIShellOptions": LMIShellOptions,
+        "LMISubscription": LMISubscription,
+        "LMIConnection": LMIConnection,
+        "LMINamespace": LMIObjectFactory().LMINamespace,
+        "LMINamespaceRoot": LMIObjectFactory().LMINamespaceRoot,
+        "LMIClass": LMIObjectFactory().LMIClass,
+        "LMIInstance": LMIObjectFactory().LMIInstance,
+        "LMIInstanceName": LMIObjectFactory().LMIInstanceName,
+        "LMIMethod": LMIObjectFactory().LMIMethod,
+        "CIMError": CIMError,
+        "ConnectionError": ConnectionError,
+        "use_exceptions": lmi_set_use_exceptions,
+        "lmi_associators": lmi_associators,
+        "lmi_isinstance": lmi_isinstance,
+        "help": LMIHelper()
     }
 
     def __init__(self, cwd_first_in_path=False):
@@ -90,9 +96,9 @@ class LMIConsole(code.InteractiveConsole):
         # should propagate caught exceptions, or dump them
         lmi_set_use_exceptions(self._use_exceptions)
 
-        # Append/Prepend current working directory into Python's search list, so we can
-        # properly import any module placed in CWD. By default, the CWD is appended for
-        # security reasons.
+        # Append/Prepend current working directory into Python's search list,
+        # so we can properly import any module placed in CWD. By default, the
+        # CWD is appended for security reasons.
         if cwd_first_in_path:
             sys.path.insert(0, os.getcwd())
         else:
@@ -112,43 +118,45 @@ class LMIConsole(code.InteractiveConsole):
 
         :param dictionary locals: locals
         """
-        def connect(uri, username="", password="", **kwargs):
+        def connect(uri, username="", password="", key_file=None,
+                    cert_file=None,
+                    verify_server_cert=self._verify_server_cert,
+                    prompt_prefix=""):
             """
-            Returns :py:class:`LMIConnection` object with provided URI and
+            Returns :py:class:`.LMIConnection` object with provided URI and
             credentials.
 
             :param string uri: URI of the CIMOM
-            :param string username: account, under which, the CIM calls will be performed
+            :param string username: account, under which, the CIM calls will be
+                performed
             :param string password: user's password
             :param dictionary kwargs: supported keyword arguments
-
-                * **key_file** (*string*) -- path to x509 key file; default value is None
-                * **cert_file** (*string*) -- path to x509 cert file; default value is None
-                * **verify_server_cert** (*bool*) -- tells, if the server side certificate
-                  needs to be verified, if SSL used; default value is True
-                * **prompt_prefix** (*string*) -- used for username/password prompt
+            :param string key_file: path to x509 key file; default value is
+                None
+            :param string cert_file: path to x509 cert file; default value is
+                None
+            :param bool verify_server_cert** (*bool*) -- tells, if the server
+                side certificate needs to be verified, if SSL used; default
+                value is True
+            :param string prompt_prefix: used for username/password prompt;
+                default value is empty string
 
             :returns: connection object
-            :rtype: :py:class:`LMIConnection`
+            :rtype: :py:class:`.LMIConnection`
             """
-            key_file = kwargs.pop("key_file", None)
-            cert_file = kwargs.pop("cert_file", None)
-            verify_server_cert = kwargs.pop("verify_server_cert", self._verify_server_cert)
-            prompt_prefix = kwargs.pop("propmpt_prefix", "")
-            if kwargs:
-                raise TypeError("connect() got an unexpected keyword arguments: %s" % ", ".join(kwargs.keys()))
-
-            return connect_internal(uri, username, password, interactive=True,
-                use_cache=self._use_cache, key_file=key_file, cert_file=cert_file,
-                verify_server_cert=verify_server_cert, prompt_prefix=prompt_prefix)
+            return connect_internal(
+                uri, username, password, interactive=True,
+                use_cache=self._use_cache, key_file=key_file,
+                cert_file=cert_file, verify_server_cert=verify_server_cert,
+                prompt_prefix=prompt_prefix)
 
         # Initialize the interpreter
         if locals is None:
             locals = {}
         else:
-            for (k, v) in locals.iteritems():
+            for k, v in locals.iteritems():
                 if isinstance(v, LMIConnection):
-                    locals[k]._client.interactive = True
+                    locals[k].client.interactive = True
         locals["clear_history"] = self.clear_history
         locals["connect"] = connect
         self.locals = dict(self.locals.items() + locals.items())
@@ -197,7 +205,8 @@ class LMIConsole(code.InteractiveConsole):
             sys.ps2 = old_sys_ps2
         self.save_history()
 
-    def interpret(self, script_name, script_argv, locals=None, interactive=False):
+    def interpret(self, script_name, script_argv, locals=None,
+                  interactive=False):
         """
         Interprets a specified script within additional provided locals.
         There are :py:attr:`LMIConsole.DEFAULT_LOCALS` present.
@@ -205,48 +214,50 @@ class LMIConsole(code.InteractiveConsole):
         :param string script_name: script name
         :param list script_argv: script CLI arguments
         :param dictionary locals: dictionary with locals
-        :param bool interactive: tells LMIShell, if the script should be treated
-            as if it was run in interactive mode
+        :param bool interactive: tells LMIShell, if the script should be
+            treated as if it was run in interactive mode
         :returns: exit code of the script
         :rtype: int
         """
         # Helper function for LMIShell's scripts
-        def connect(uri, username="", password="", **kwargs):
+        def connect(uri, username="", password="", key_file=None,
+                    cert_file=None,
+                    verify_server_cert=self._verify_server_cert,
+                    prompt_prefix=""):
             """
-            Returns :py:class:`LMIConnection` object with provided URI and credentials.
+            Returns :py:class:`.LMIConnection` object with provided URI and
+            credentials.
 
             :param string uri: URI of the CIMOM
-            :param string username: account, under which, the CIM calls will be performed
+            :param string username: account, under which, the CIM calls will be
+                performed
             :param string password: user's password
             :param dictionary kwargs: supported keyword arguments
-
-                * **key_file** (*string*) -- path to x509 key file; default value is None
-                * **cert_file** (*string*) -- path to x509 cert file; default value is None
-                * **verify_server_cert** (*bool*) -- tells, if the server side certificate
-                  needs to be verified, if SSL used; default value is True
-                * **prompt_prefix** (*string*) -- used for username/password prompt
+            :param string key_file: path to x509 key file; default value is
+                None
+            :param string cert_file: path to x509 cert file; default value is
+                None
+            :param bool verify_server_cert: tells, if the server side
+                certificate needs to be verified, if SSL used; default value is
+                True
+            :param string prompt_prefix: used for username/password prompt;
+                default value is empty string
 
             :returns: connection object
-            :rtype: :py:class:`LMIConnection`
+            :rtype: :py:class:`.LMIConnection`
             """
-            # Set remaining arguments
-            key_file = kwargs.pop("key_file", None)
-            cert_file = kwargs.pop("cert_file", None)
-            verify_server_cert = kwargs.pop("verify_server_cert", self._verify_server_cert)
-            prompt_prefix = kwargs.pop("prompt_prefix", "")
-            if kwargs:
-                raise TypeError("connect() got an unexpected keyword arguments: %s" % ", ".join(kwargs.keys()))
-
-            return connect_internal(uri, username, password, interactive=interactive,
-                use_cache=self._use_cache, key_file=key_file, cert_file=cert_file,
-                verify_server_cert=verify_server_cert, prompt_prefix=prompt_prefix)
+            return connect_internal(
+                uri, username, password, interactive=interactive,
+                use_cache=self._use_cache, key_file=key_file,
+                cert_file=cert_file, verify_server_cert=verify_server_cert,
+                prompt_prefix=prompt_prefix)
         # Initialize locals
         if locals is None:
             locals = {}
         else:
-            for (k, v) in locals.iteritems():
+            for k, v in locals.iteritems():
                 if isinstance(v, LMIConnection):
-                    locals[k]._client.interactive = False
+                    locals[k].client.interactive = False
         locals["connect"] = connect
         self.locals = dict(self.locals.items() + locals.items())
 
@@ -268,15 +279,17 @@ class LMIConsole(code.InteractiveConsole):
         if self._history_length == 0 or not os.path.exists(self._history_file):
             return
         readline.read_history_file(self._history_file)
-        if self._history_length > 0 and readline.get_current_history_length() > self._history_length:
+        if self._history_length > 0 and \
+                readline.get_current_history_length() > self._history_length:
             readline.set_history_length(self._history_length)
             readline.write_history_file(self._history_file)
             readline.read_history_file(self._history_file)
 
     def save_history(self):
         """
-        Saves current history of commands into the history file. If the length of history
-        exceeds a maximum history file length, the history will be truncated.
+        Saves current history of commands into the history file. If the length
+        of history exceeds a maximum history file length, the history will be
+        truncated.
         """
         if self._history_length == 0:
             return
@@ -297,7 +310,7 @@ class LMIConsole(code.InteractiveConsole):
         """
         Turns on or off server side certificate verification, if SSL used.
 
-        :param bool verify_server_cert: -- flag which tells, whether a server side
-            certificate needs to be verified, if SSL used
+        :param bool verify_server_cert: -- flag which tells, whether a server
+            side certificate needs to be verified, if SSL used
         """
         self._verify_server_cert = verify_server_cert
