@@ -13,6 +13,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
+
 class LMIObjectFactory(object):
     """
     Object factory class. Used to avoid circular import dependencies between
@@ -34,16 +35,14 @@ class LMIObjectFactory(object):
 
         :param string name: member name, which should be returned
         """
-        if name in self.__dict__:
-            return self.__dict__[name]
         if name in self._classes:
             return self._classes[name]
         raise AttributeError(name)
 
     def __new__(cls):
         """
-        Creates a new :py:class:`LMIObjectFactory` instance, if not created, and returns
-        the singleton instance object.
+        Creates a new :py:class:`.LMIObjectFactory` instance, if not created,
+        and returns the singleton instance object.
         """
         if not cls._instance:
             cls._instance = super(LMIObjectFactory, cls).__new__(cls)
